﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Net.Http;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Text;
using ServiceStack.Logging;
using ServiceStack.Serialization;

namespace Aggregator.Aggregation.CarreApi
{
    public class ApiClient : IDisposable
    {
        private readonly HttpClient _httpClient;
	    private Dictionary<string, User> _userCache;
	    private ILog logger;

        public ApiClient()
        {
            _httpClient = new HttpClient {BaseAddress = new Uri("https://carre.kmi.open.ac.uk")};
			_userCache = new Dictionary<string, User>();
			logger = LogManager.GetLogger(GetType());
        }

	    public bool HasDataTypeMapped(BiometricHealthDataType type)
	    {
		    return SparqlQueryHelper.BiometricDatatypes.ContainsKey(type);
	    }

	    public bool HasDataTypeMapped(DiagnosisHealthDataType type)
	    {
		    return SparqlQueryHelper.DiagnosisDatatypes.ContainsKey(type);
	    }

	    public bool HasDataTypeMapped(MedicationHealthDataType type)
	    {
		    return false;
	    }

        public void UploadData(BiometricHealthDataType dataType, string data, string carreToken, DateTime date)
        {
            if (data == null)
                return;

	        var queryHelper = new SparqlQueryHelper(GetUser(carreToken));

	        var query = queryHelper.PrepareInsertQuery(new ApiRequest(dataType, data, date));

			var request = new QueryRequest {
				Sparql = query,
				Token = carreToken
			};

	        var requestString = new JsonDataContractSerializer().SerializeToString(request);

			var content = new StringContent(requestString, Encoding.UTF8, "application/json");

			var result = _httpClient.PostAsync("/ws/query", content).Result;

	        if (result.IsSuccessStatusCode)
	        {
				logger.Info("SUCCESS Uploaded data: " + requestString);
		        return;
	        }
	        else
	        {
		        logger.Info("FAILED Uploaded data: " + requestString);
	        }

	        throw new ApiException();
        }

	    public void UploadData(DiagnosisHealthDataType dataType, string data, string carreToken, DateTime date)
	    {
			if (data == null)
				return;

			var queryHelper = new SparqlQueryHelper(GetUser(carreToken));

			var query = queryHelper.PrepareInsertQuery(new ApiRequest(dataType, data, date));

			var request = new QueryRequest {
				Sparql = query,
				Token = carreToken
			};

		    var requestString = new JsonDataContractSerializer().SerializeToString(request);

			var content = new StringContent(requestString, Encoding.UTF8, "application/json");

			var result = _httpClient.PostAsync("/ws/query", content).Result;

			if (result.IsSuccessStatusCode) {
				logger.Info("SUCCESS Uploaded data: " + requestString);
				return;
			} else {
				logger.Info("FAILED Uploaded data: " + requestString);
			}

			throw new ApiException();   
	    }

	    private User GetUser(string token)
	    {
		    if (_userCache.ContainsKey(token))
		    {
			    return _userCache[token];
		    }

		    using (var client = new HttpClient()) {
				client.BaseAddress = new Uri(ConfigurationManager.AppSettings["CarreBaseUrl"]);

				var result = client.GetAsync("/ws/userProfile?token=" + token).Result;
				var resultContent = result.Content.ReadAsStringAsync().Result;

				var serializer = new JsonDataContractSerializer();

				var userProfile = serializer.DeserializeFromString<User>(resultContent);

			    _userCache.Add(token, userProfile);

				return userProfile;
			}    
	    }

	    public void SendConnectionCallback(ConnectionCallback callback)
	    {
			var httpClient = new HttpClient();

			var serializer = new DataContractJsonSerializer(typeof(ConnectionCallback));

			var ms = new MemoryStream();
			serializer.WriteObject(ms, callback);
			var serialized = Encoding.Default.GetString(ms.ToArray());

		    var response =
			    httpClient.PostAsync(new Uri("https://carre.kmi.open.ac.uk/devices/oauth/phr/callback"),
				    new StringContent(serialized, Encoding.UTF8, "application/json")).Result;

		    if (response.IsSuccessStatusCode)
		    {
				logger.Warn("SUCCESS Callback : " + serialized);
		    }
		    else
		    {
			    logger.Error("ERROR Callback : " + serialized);
				logger.Error("CALLBACK LOCATION : " + response.Headers.Location);
			    logger.Error("CALLBACK RESPONSE STATUS : " + response.StatusCode);
				logger.Error("CALLBACK RESPONSE : " + response.Content.ReadAsStringAsync().Result);
		    }
	    }

        public void Dispose()
        {
            _httpClient.Dispose();
        }

		[DataContract]
		class QueryRequest
		{
			[DataMember(Name = "sparql")]
			public string Sparql { get; set; }

			[DataMember(Name = "token")]
			public string Token { get; set; }
		}
    }
}