﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using Microsoft.Health;
using Microsoft.Health.ItemTypes;
using Microsoft.Health.Web;

namespace Aggregator.Aggregation.PublicHealthRepositoryApi.Clients.HealthVault
{
	public class HealthVaultApiClient : IPublicHealthRepositoryApiClient
	{
		public readonly Dictionary<BiometricHealthDataType, Guid> UrlBiometricHealthDataTypeDictionary;
		public readonly Dictionary<DiagnosisHealthDataType, Guid> UrlDiagnosisHealthDataTypeDictionary;

		public HealthVaultApiClient()
		{
			#region data maps 

			UrlBiometricHealthDataTypeDictionary = new Dictionary<BiometricHealthDataType, Guid>
			{
				{ BiometricHealthDataType.SystolicBloodPressure, BloodPressure.TypeId},
				{ BiometricHealthDataType.DiastolicBloodPressure, BloodPressure.TypeId},
				{ BiometricHealthDataType.HeartRate, BloodPressure.TypeId},
				{ BiometricHealthDataType.TotalCholesterol, BloodPressure.TypeId},
				{ BiometricHealthDataType.TriglyceridesSerumConcentration, BloodPressure.TypeId},
				{ BiometricHealthDataType.UricAcidSerumConcentration, BloodPressure.TypeId},
				//{ BiometricHealthDataType.DiastolicBloodPressure, BloodPressure.TypeId},
			};

			UrlDiagnosisHealthDataTypeDictionary = new Dictionary<DiagnosisHealthDataType, Guid>()
			{
				
			};

			#endregion
		}
		
		public IList<HealthDataDto> GetHealthData(BiometricHealthDataType type, string token, DateTime @from)
		{
			var list = new List<HealthDataDto>();

			var _connection = new OfflineWebApplicationConnection(//clientID,
				new Guid(ConfigurationManager.AppSettings["HealthVaultAppId"]), //Properties.Settings.Default.MasterAppGUID,
				new Uri(ConfigurationManager.AppSettings["HealthVaultPlatform"]),
				new Guid(token));

			_connection.Authenticate();
			
			var tokenGuid = new Guid(token);

			var person = _connection.GetAuthorizedPeople().FirstOrDefault(p => p.PersonId == tokenGuid);

			if (person == null)
			{
				return list;
			}

			var selfRecord = person.GetSelfRecord();

			if (type == BiometricHealthDataType.SystolicBloodPressure)
			{
				return GetSystolicBloodPressure(selfRecord);
			}
			if (type == BiometricHealthDataType.DiastolicBloodPressure)
			{
				return GetDiastolicBloodPressure(selfRecord);
			}
			if (type == BiometricHealthDataType.HeartRate)
			{
				return GetHeartRate(selfRecord);
			}
			if (type == BiometricHealthDataType.TotalCholesterol) {
				return GetTotalCholesterol(selfRecord);
			}
			if (type == BiometricHealthDataType.TriglyceridesSerumConcentration) {
				return GetTriglyceridesSerumConcentration(selfRecord);
			}
			if (type == BiometricHealthDataType.UricAcidSerumConcentration) {
				return GetUricAcid(selfRecord);
			}

			return list;
		}

		public IList<HealthDataDto> GetDiagnosisHealthData(string token, DateTime @from)
		{
			var list = new List<HealthDataDto>();

			return list;
		}

		public IList<HealthDataDto> GetMedicationHealthData(string token, DateTime @from)
		{
			//not implemented
			return new List<HealthDataDto>();
		}

		public bool DataMapExists(BiometricHealthDataType type)
		{
			return UrlBiometricHealthDataTypeDictionary.ContainsKey(type);
		}

		public bool DataMapExists(DiagnosisHealthDataType type)
		{
			return UrlDiagnosisHealthDataTypeDictionary.ContainsKey(type);
		}

		public bool DataMapExists(MedicationHealthDataType type)
		{
			throw new NotImplementedException();
		}

		private List<HealthDataDto> GetSystolicBloodPressure(HealthRecordInfo selfRecord)
		{
			var items = selfRecord.GetItemsByType(BloodPressure.TypeId);

			return (from BloodPressure bloodPressure in items 
					select new HealthDataDto(bloodPressure.Systolic.ToString(), bloodPressure.EffectiveDate)).ToList();
		}

		private List<HealthDataDto> GetDiastolicBloodPressure(HealthRecordInfo selfRecord)
		{
			var items = selfRecord.GetItemsByType(BloodPressure.TypeId);

			return (from BloodPressure bloodPressure in items
					select new HealthDataDto(bloodPressure.Diastolic.ToString(), bloodPressure.EffectiveDate)).ToList();
		}

		private List<HealthDataDto> GetHeartRate(HealthRecordInfo selfRecord)
		{
			var items = selfRecord.GetItemsByType(HeartRate.TypeId);
			var healthData = new List<HealthDataDto>();

			foreach (var item in items)
			{
				var heartRate = (HeartRate) item;
				healthData.Add(new HealthDataDto(heartRate.Value.ToString(), heartRate.EffectiveDate));				
			}

			return healthData;
		}

		private List<HealthDataDto> GetTotalCholesterol(HealthRecordInfo selfRecord)
		{
			var items = selfRecord.GetItemsByType(CholesterolProfile.TypeId);
			var healthData = new List<HealthDataDto>();

			foreach (var item in items) {
				var cholesterol = (CholesterolProfile)item;
				healthData.Add(new HealthDataDto(cholesterol.TotalCholesterol.ToString(), cholesterol.EffectiveDate));
			}

			return healthData;
		}

		private List<HealthDataDto> GetTriglyceridesSerumConcentration(HealthRecordInfo selfRecord)
		{
			var items = selfRecord.GetItemsByType(CholesterolProfileV2.TypeId);
			var healthData = new List<HealthDataDto>();

			foreach (var item in items) {
				var cholesterol = (CholesterolProfileV2)item;
				healthData.Add(new HealthDataDto(cholesterol.Triglyceride.ToString(), cholesterol.EffectiveDate));
			}

			return healthData;
		}

		private List<HealthDataDto> GetUricAcid(HealthRecordInfo selfRecord)
		{
			var items = selfRecord.GetItemsByType(LabTestResults.TypeId);
			var healthData = new List<HealthDataDto>();

			foreach (var item in items) {
				var labTestResults = (LabTestResults)item;

				foreach (var group in labTestResults.Groups)
				{
					foreach (var result in group.Results)
					{
						if (result.ClinicalCode.Any(c => c.Value == "25002500"))
						{
							healthData.Add(new HealthDataDto(result.Value.ToString(), DateTime.Parse(result.When.ApproximateDate.ToString())));							
						}						
					}	
				}
			}

			return healthData;
		}

		private List<HealthDataDto> GetAcuteKidneyInjuryDiagnosis(HealthRecordInfo selfRecord)
		{
			var items = selfRecord.GetItemsByType(Problem.TypeId);
			var healthData = new List<HealthDataDto>();

			foreach (var item in items)
			{
				var diagnosis = (Problem) item;

				if (diagnosis.Diagnosis.Any(d => AggregatorDictionaries.DiagnosisTypeToIcdDictionary[DiagnosisHealthDataType.AcuteKidneyInjuryDiagnosis].Contains(d.Text)))
				{
					var diagnoze =
						diagnosis.Diagnosis.FirstOrDefault(
							d =>
								AggregatorDictionaries.DiagnosisTypeToIcdDictionary[DiagnosisHealthDataType.AcuteKidneyInjuryDiagnosis].Contains
									(d.Text));

					healthData.Add(new HealthDataDto(diagnoze.Text, diagnosis.When.ToDateTime()));
				}
			}

			return healthData;
		} 
	}
}